/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014 (c)
 * 
 *    file: custom_func.c
 *    This file is part of tcalc.
 *
 *    tcalc is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    tcalc is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with tcalc.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef __CUSTOM_FUNCS
#define __CUSTOM_FUNCS
#define MAX_FUNCTIONS		20
#define MAX_FUNCTION_NAME	10
#define MAX_VARIABLES		10
#define MAX_VARIABLE_LEN	10
#endif /* __CUSTOM_FUNCS */

#ifndef __FORMATS
#define __FORMATS
#define DECIMAL_FORMAT	1
#define HEX_FORMAT	2
#define BINARY_FORMAT	3
#define OCTAL_FORMAT	4
#define ALL_FORMATS	5
#endif /* __FORMATS */

//number format functions
extern long hex_to_decimal(char *hex);
extern long binary_to_decimal(char *binary);
extern long octal_to_decimal(char *octal);
extern void decimal_to_hex(long decimal, char **res);
extern void decimal_to_binary(long decimal, char **res);
//extern long decimal_to_octal(long decimal);
extern void convert_numbers(char *input);

char custom_func_name[MAX_FUNCTIONS][MAX_FUNCTION_NAME];
char *func_body[MAX_FUNCTIONS];
char *func_args[MAX_FUNCTIONS];
int func_args_count[MAX_FUNCTIONS];
int func_body_len[MAX_FUNCTIONS];
char var_name[MAX_VARIABLES][MAX_VARIABLE_LEN];
float var_value[MAX_VARIABLES];
int total_vars;
int total_custom_funcs = 0;

extern double process_sub_formula_f(char *subformula, int end);
extern void print_mult_table(char *n);
//standard math functions
extern double process_math_func(char *func, int ob_index, int cb_index);

void print_custom_funcs()
{
  int i;
  printf("Custom-defined functions:\n");
  if(!total_custom_funcs)
    printf("  No functions defined\n");
  else
    for(i = 0; i < total_custom_funcs; i++)
      printf("  %s(%s)\n", custom_func_name[i], func_args[i]);
}


int process_custom_func(char *name, char *str, char *args, double *res)
{
  int i;
  int k;
  char *tmp;
  
  //char END_BRACE = 0;
  int func_index = -1;
  strcpy(var_name[0], name);
  total_vars = 1;
  for(i = 0; i < total_custom_funcs; i++)
  {
    if(strcmp(name, custom_func_name[i]) == 0)
    {
      func_index = i;
      break;
    }
  }
  if(func_index < 0)
  {
    printf("Error: Undefined function: %s\n", name);
    return 0;
  }
  //extract function argument names, if any
  if(func_args_count[func_index])
  {
    int l;
    k = l = 0;
    i = 1;
    while(k < strlen(func_args[func_index]))
    {
      while(func_args[func_index][k] != ',' && k < strlen(func_args[func_index]))
	k++;
      strncpy(var_name[i], func_args[func_index]+l, k-l);
      var_name[i][k-l] = '\0';
      if(k < strlen(func_args[func_index]))
      {
	k++;
	while(func_args[func_index][k] == ' ') k++;
	l = k;
      }
      //printf("var %d = %s\n", i, var_name[i]);
      i++;
    }
    func_args_count[func_index] = i-1;
    total_vars = i;
  }
  
  //extract passed arguments, if any
  if(args)
  {
    int l;
    k = l = 0;
    i = 1;
    char tmp_var[10];
    while(k < strlen(args))
    {
      while(args[k] != ',' && k < strlen(args))
	k++;
      strncpy(tmp_var, args+l, k-l);
      tmp_var[k-l] = '\0';
      var_value[i] = atof(tmp_var);
      if(k < strlen(args))
      {
	k++;
	while(args[k] == ' ') k++;
	l = k;
      }
      i++;
    }
    i--;
    if(i != func_args_count[func_index])
    {
      printf("Error: Missing argument(s) to function %s\n", name);
      return 0;
    }
  }

  int active_var = 0;
  
  tmp = strtok(str, "\n");
  if(!tmp)
  {
    printf("Error: Invalid function body\n");
    return 0;
  }
  //printf("%s\n", tmp);
  //int m;
  //for(m = 0; m < strlen(tmp); m++)
    //printf("%c [%d]\n", tmp[m], tmp[m]);
  goto parse_line;
  
  while((tmp = strtok(NULL, "\n")))
  {
parse_line:
    if(0) ;	//to shut up gcc
    char COMMENT = 0;
    for(i = 0; i < strlen(tmp); i++)
      if(tmp[i] == ' ' || tmp[i] == '\t') i++;
      else break;
    if(tmp[i] == '#') { COMMENT = 1; continue; }
    /*if(tmp[0] == '}')
      END_BRACE = 1;*/
    
    //////////////////////////////
    //variable declaration
    //////////////////////////////
    if(strstr(tmp, "var"))
    {
      if(total_vars >= MAX_VARIABLES)
      {
	printf("Error: maximum number of variables exceeded.\n");
	return 0;
      }
      char *tmp2 = strstr(tmp, "var");
      int l = 4;
      while(tmp2[l] == ' ') l++;
      int var_len = (strlen(tmp)-l);
      if(var_len > MAX_VARIABLE_LEN)
      {
	printf("Error: maximum length of variable name exceeded.\n");
	return 0;
      }
      strncpy(var_name[total_vars], tmp2+l, var_len);
      total_vars++;
      //free(tmp2);
      continue;
    }
    ////////////////////////
    ////////////////////////
    else if(strstr(tmp, "convert"))
    {
      int j;
      for(j = 7; j < strlen(tmp); j++)
	if(tmp[j] != ' ' && tmp[j] != '\n' && tmp[j] != '\t')
	{ convert_numbers(tmp+j); break; }
      continue;
    }
    else if(strstr(tmp, "table"))
    {
      int j;
      for(j = 5; j < strlen(tmp); j++)
	if(tmp[j] != ' ' && tmp[j] != '\n' && tmp[j] != '\t')
	{ print_mult_table(tmp+j); break; }
      continue;
    }
    //////////////////
    //return clause
    //////////////////
    else if(strstr(tmp, "return"))
    {
      char *tmp2 = strstr(tmp, "return");
      int l = 6;
      while(tmp2[l] == ' ') l++;
      
      for(i = 0; i < total_vars; i++)
      {
	if(strncmp(tmp2+l, var_name[i], strlen(var_name[i])) == 0)
	{
	  if((tmp2[l+strlen(var_name[i])] != '\n')
	     && (tmp2[l+strlen(var_name[i])] != '\0')) break;
	  //free(tmp2);
	  *res = var_value[i];
	  return 1;
	}
      }
      active_var = 0;
      k = l; i = k+1;
      goto processing;
    }
    //////////////////////////////
    //process other calculations
    //////////////////////////////
    i = 0;
    double result_f = 0;
    char op = '.';
    char *tmp2;
    for(k = 0; k <= strlen(tmp); k++)
    {
processing:
      if(tmp[k] == '#') { COMMENT = 1; break; }
      else if(tmp[k] == '(')
      {
	if(!strrchr(tmp, ')'))
	{
	  fprintf(stderr, "Error: Missing ')'\n");
	  result_f = -1; break;
	}
	int j;
	int open_braces = 1, close_braces = 0;
	for(j = k+1; j < strlen(tmp); j++)
	{
	  if(tmp[j] == ')') close_braces++;
	  else if(tmp[j] == '(') open_braces++;
	  if(open_braces == close_braces) break;
	}
	double tmp_res = 
	  process_sub_formula_f(tmp+k, j-k);
	switch(op)
	{
	    case '+': result_f += tmp_res; break;
	    case '-': result_f -= tmp_res; break;
	    case '*': result_f *= tmp_res; break;
	    case '/': result_f /= tmp_res; break;
	    case '%': result_f = (int)result_f%(int)tmp_res; break;
	    case '.': result_f = tmp_res; break;
	}
	k = j;
	op = tmp[k];
	i = k+1;
      }
      else if(tmp[k] == '+' || tmp[k] == '-' ||
	      tmp[k] == '*' || tmp[k] == '/' ||
	      tmp[k] == '%' || tmp[k] == '\0'||
	      tmp[k] == '\n' || tmp[k] == '}')
      {
	tmp2 = strndup(tmp+i, k-i);
	switch(op)
	{
	    case '+':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f += binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f += hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f += octal_to_decimal(tmp2);
	      else result_f += atof(tmp2);
	      break;
	    case '-':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f -= binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f -= hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f -= octal_to_decimal(tmp2);
	      else result_f -= atof(tmp2);
	      break;
	    case '*':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f *= binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f *= hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f *= octal_to_decimal(tmp2);
	      else result_f *= atof(tmp2);
	      break;
	    case '/':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f /= binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f /= hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f /= octal_to_decimal(tmp2);
	      else result_f /= atof(tmp2);
	      break;
	    case '%':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f = (int)result_f%binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f = (int)result_f%hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f = (int)result_f%octal_to_decimal(tmp2);
	      else result_f = (int)result_f%atoi(tmp2);
	      break;
	    case '.':
	      if(tmp2[0] == 'b' || tmp2[0] == 'B')
		result_f = binary_to_decimal(tmp2+1);
	      else if((tmp2[0] == '0' && tmp2[1] == 'X') ||
		      (tmp2[0] == '0' && tmp2[1] == 'x'))
		result_f = hex_to_decimal(tmp2);
	      else if(tmp2[0] == '0')
		result_f = octal_to_decimal(tmp2);
	      else result_f = atof(tmp2);
	      break;
	    /*case '+': result_f += atof(tmp2); break;
	    case '-': result_f -= atof(tmp2); break;
	    case '*': result_f *= atof(tmp2); break;
	    case '/': result_f /= atof(tmp2); break;
	    case '%': result_f = (int)result_f%(int)atof(tmp2); break;
	    case '.': result_f = atof(tmp2); break;*/
	}
	op = tmp[k];
	i = k+1;
      }      
      ///////////////////////////////////////
      //process math functions
      ///////////////////////////////////////
      else if(tmp[k] >= 'a' && tmp[k] <= 'z')
      {
	char *ob = strchr(tmp+k, '(');
	char *cb = strchr(tmp+k, ')');
	char *eq = strchr(tmp+k, '=');
	if(!ob || !cb || (eq && (eq < ob)))
	{
	  //seek for variable assignment
	  if(strchr(tmp+k, '='))
	  {
	    int j;
	    for(j = k; j < strlen(tmp); j++)
	      if(tmp[j] == ' ' || tmp[j] == '=') break;
	    char *v = strndup(tmp+k, j-k);
	    while(tmp[j] != '=') j++;
	    k = j+1;
	    i = k+1;
	    op = '.';
	    for(j = 0; j < total_vars; j++)
	      if(strcmp(var_name[j], v) == 0)
	      { active_var = j; break; }
	    if(j == total_vars)
	    {
	      printf("Variable %s not defined\n", v);
	      free(v);
	      result_f = -1; break;
	    }
	    free(v);
	  }
	  else
	  {
	    while(tmp[k] >= 'a' && tmp[k] <= 'z') k++;
	    char *v2 = strndup(tmp+i, k-i);
	    int j;
	    for(j = 0; j < total_vars; j++)
	      if(strcmp(var_name[j], v2) == 0)
	      {
		switch(op)
		{
		  case '+': result_f += var_value[j]; break;
		  case '-': result_f -= var_value[j]; break;
		  case '*': result_f *= var_value[j]; break;
		  case '/': result_f /= var_value[j]; break;
		  case '%': result_f = (int)result_f%(int)var_value[j]; break;
		  case '.': result_f = var_value[j]; break;
		}
		break;
	      }
	    i = k+1;
	    op = tmp[k];
	    free(v2);
	    /*if(!ob) printf("Error: missing '('\n");
	    else    printf("Error: missing ')'\n");
	    result_f = -1; break;*/
	  }
	}
	else
	{
	  double r =
	    process_math_func(tmp+k, ob-(tmp+k), cb-(tmp+k));
	  if(r == -1) break;
	  switch(op)
	  {
	    case '+': result_f += r; break;
	    case '-': result_f -= r; break;
	    case '*': result_f *= r; break;
	    case '/': result_f /= r; break;
	    case '%': result_f = (int)result_f%(int)r; break;
	    case '.': result_f = r; break;
	  }
	  k = cb-(tmp+k)+1;
	  op = tmp[k];
	  i = k+1;
	  free(ob); free(cb);
	}
      }
      else if(tmp[k] == ' ' || tmp[k] == '\t') i++;
    }//end for
    if(!COMMENT)
      var_value[active_var] = result_f;
  }//end while
  return 1;
} /* process_custom_func() */