/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2014 (c)
 * 
 *    file: input_free.c
 *    This file is part of tcalc.
 *
 *    tcalc is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    tcalc is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with tcalc.  If not, see <http://www.gnu.org/licenses/>.
 */    

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

extern const char tcalc_ver[];
extern int number_mode;
extern int decimal_point, input_format, output_format;
#ifndef __NUMBER_MODES
#define __NUMBER_MODES
#define NUMBER_INT	1
#define NUMBER_FLOAT	2
#endif /* __NUMBER_MODES */

#ifndef __FORMATS
#define __FORMATS
#define DECIMAL_FORMAT	1
#define HEX_FORMAT	2
#define BINARY_FORMAT	3
#define OCTAL_FORMAT	4
#define ALL_FORMATS	5
#endif /* __FORMATS */

//number format functions
extern long hex_to_decimal(char *hex);
extern long binary_to_decimal(char *binary);
extern long octal_to_decimal(char *octal);
extern void decimal_to_hex(long decimal, char **res);
extern void decimal_to_binary(long decimal, char **res);
//extern long decimal_to_octal(long decimal);
extern void convert_numbers(char *input);
//standard math functions
extern double process_math_func(char *func, int ob_index, int cb_index);
extern void print_func_descriptions();
extern int total_funcs;
extern char *func_name[];
//custom functions
#ifndef __CUSTOM_FUNCS
#define __CUSTOM_FUNCS
#define MAX_FUNCTIONS		20
#define MAX_FUNCTION_NAME	10
#define MAX_VARIABLES		10
#define MAX_VARIABLE_LEN	10
#endif /* __CUSTOM_FUNCS */

extern char custom_func_name[][MAX_FUNCTION_NAME];
extern char *func_body[];
extern char *func_args[];
extern int func_args_count[];
extern int func_body_len[];
extern char var_name[];
extern float var_value[];
extern int total_vars;
extern int total_custom_funcs;
extern void print_custom_funcs();

extern double process_sub_formula_f(char *subformula, int end);
extern void print_mult_table(char *n);
extern void input_from_file_mode(FILE *file);
void show_help();


void input_free_mode()
{
    char formula[1024];
    formula[1023] = '\0';
    while(1)
    {
	  //char COMMENT = 0;
	  //scanf("%s", formula);
	  printf(">");
	  fgets(formula, 1023, stdin);
	  //printf("%s", formula);
	  //char ccc = getchar();
	  //printf("%s, %s\n", formula, strstr(formula, "output"));
	  if(formula[0] == '\n') continue;
	  if(strcmp(formula, "end\n") == 0) break;
	  
	  int j;
	  for(j = 0; j < strlen(formula); j++)
	    if(formula[j] == ' ' || formula[j] == '\t') j++;
	    else break;
	  if(formula[j] == '#') continue;
	  
	  if(strcmp(formula, "q\n") == 0 || strcmp(formula, "Q\n") == 0 ||
	     strcmp(formula, "quit\n") == 0) break;
	  else if(strcmp(formula, "ver\n") == 0 || strcmp(formula, "version\n") == 0)
	  {
	    printf("Version %s\n", tcalc_ver); continue;
	  }
	  else if(strcmp(formula, "h\n") == 0 || strcmp(formula, "help\n") == 0)
	  {
	    show_help(); continue;
	  }
	  else if(strcmp(formula, "math\n") == 0)
	  {
	    print_func_descriptions(); continue;
	  }
	  else if(strcmp(formula, "funcs\n") == 0)
	  {
	    print_custom_funcs(); continue;
	  }
	  else if(strstr(formula, "input"))
	  {
	    int l = strstr(formula, "input")-formula+5;
	    while(formula[l] == ' ') l++;
	    FILE *file;
	    if((file = fopen(formula+l, "rb")) == NULL)
	    {
	      fprintf(stderr, "Error opening file: %s\n", formula+l);
	    }
	    printf("Reading from %s\n", formula+l);
	    input_from_file_mode(file);
	    fclose(file);
	    continue;
	  }
	  //////////////////////////////////////
	  //////////////////////////////////////
	  else if(strstr(formula, "func"))
	  {
	    int l = strstr(formula, "func")-formula+4;
	    while(formula[l] == ' ') l++;
	    int l2 = l+1;
	    while(formula[l2] >= 'a' && formula[l2] <= 'z') l2++;
	    //l2++;
	    
	    int j;
	    char ERROR = 0;
	    for(j = 0; j < total_funcs; j++)
	    {
	      if(strncmp(func_name[j], formula+l, strlen(func_name[j])) == 0)
	      {
		printf("Error: %s is a reserved function\n", func_name[j]);
		ERROR = 1; break;
	      }
	    }
	    if(ERROR) continue;
	    for(j = 0; j < total_custom_funcs; j++)
	    {
	      if(strncmp(custom_func_name[j], formula+l, strlen(custom_func_name[j])) == 0)
	      {
		printf("Error: %s is already defined\n", custom_func_name[j]);
		ERROR = 1; break;
	      }
	    }
	    if(ERROR) continue;
	    /*custom_func_name[total_custom_funcs] = (char *)malloc(l2-l);
	    if(!custom_func_name[total_custom_funcs])
	    {
	      printf("Error: Insufficient memory\n"); continue;
	    }*/
	    strncpy(custom_func_name[total_custom_funcs], formula+l, l2-l);
	    custom_func_name[total_custom_funcs][l2-l] = '\0';
	    //extract function arguments, if any
	    func_args_count[total_custom_funcs] = 0;
	    l = 0;
	    int l3 = 0;
	    for(j = l2; j < strlen(formula); j++)
	    {
	      if(formula[j] == '(') l = j+1;
	      if(formula[j] == ')') l3 = j-1;
	    }
	    if(!l3)
	    {
		printf("Error: Missing ')'\n");
		continue;
	    }
	    /*func_args[total_custom_funcs] = (char *)malloc(l3-l+1);
	    if(!func_args[total_custom_funcs])
	    {
	      printf("Error: Insufficient memory\n");
	      continue;
	    }*/
	    if(l)
	    {
	      func_args[total_custom_funcs] = strndup(formula+l, l3-l+1);
	      func_args_count[total_custom_funcs] = 1;
	    }
	    func_body[total_custom_funcs] = (char *)malloc(1);
	    func_body_len[total_custom_funcs] = 0;
	    while(1)
	    {
	      printf("->");
	      fgets(formula, 1023, stdin);
	      if(strcmp(formula, "q\n") == 0) break;
	      func_body_len[total_custom_funcs] += strlen(formula);
	      func_body[total_custom_funcs] = (char *)
			realloc(func_body[total_custom_funcs],
				func_body_len[total_custom_funcs]);
	      if(!func_body[total_custom_funcs])
	      {
		printf("Error: Insufficient memory\n");
		//free(custom_func_name[total_custom_funcs]);
		free(func_body[total_custom_funcs]);
		ERROR = 1; break;
	      }
	      strcat(func_body[total_custom_funcs], formula);
	      if(strchr(formula, '}')) break;
	    }
	    if(ERROR) continue;
	    total_custom_funcs++; continue;
	  }
	  //////////////////////////////////////
	  //////////////////////////////////////
	  else if(strstr(formula, "decimal"))
	  {
	    decimal_point = atoi(strchr(formula, '=')+1); continue;
	  }
	  else if(strstr(formula, "format"))
	  {
	    if(!strchr(formula, '='))
	    { printf("Error: Invalid command\n"); continue; }
	    char *f = strchr(formula, '=');
	    if(strstr(f, "d") || strstr(f, "decimal"))
	      input_format = DECIMAL_FORMAT;
	    else if(strstr(f, "x") || strstr(f, "hex"))
	      input_format = HEX_FORMAT;
	    else if(strstr(f, "b") || strstr(f, "binary"))
	      input_format = BINARY_FORMAT;
	    else if(strstr(f, "o") || strstr(f, "octal"))
	      input_format = OCTAL_FORMAT;
	    else
	    {
	      printf("Error: Unknown number format\n");
	    }
	    //free(f);
	    continue;
	  }
	  else if(strstr(formula, "output"))
	  {
	    //printf("%s\n", strchr(formula, '='));
	    if(!strchr(formula, '='))
	    { printf("Error: Invalid command\n"); continue; }
	    char *f = strchr(formula, '=');
	    if(strstr(f, "d") || strstr(f, "decimal"))
	      output_format = DECIMAL_FORMAT;
	    else if(strstr(f, "x") || strstr(f, "hex"))
	      output_format = HEX_FORMAT;
	    else if(strstr(f, "b") || strstr(f, "binary"))
	      output_format = BINARY_FORMAT;
	    else if(strstr(f, "o") || strstr(f, "octal"))
	      output_format = OCTAL_FORMAT;
	    else if(strstr(f, "a") || strstr(f, "all"))
	      output_format = ALL_FORMATS;
	    else
	    {
	      printf("Error: Unknown number format\n");
	    }
	    //printf("fmt=%s\n", f);
	    //free(f);
	    continue;
	  }
	  else if(strstr(formula, "convert"))
	  {
	    int j;
	    for(j = 7; j < strlen(formula); j++)
	      if(formula[j] != ' ' && formula[j] != '\n' && formula[j] != '\t')
	      { convert_numbers(formula+j); break; }
	    continue;
	  }
	  else if(strstr(formula, "table"))
	  {
	    int j;
	    for(j = 5; j < strlen(formula); j++)
	      if(formula[j] != ' ' && formula[j] != '\n' && formula[j] != '\t')
	      { print_mult_table(formula+j); break; }
	    continue;
	  }
	  
	  int i = 0, k;
	  char *tmp;
	  long result = 0;
	  double result_f = 0;
	  char op = '.';
	  number_mode = 0;
	  //while(i < strlen(formula))
	  //{
	    for(k = 0; k <= strlen(formula); k++)
	    {
	      if(formula[k] == '(')
	      {
		if(!strrchr(formula, ')'))
		{
		  fprintf(stderr, "Error: Missing ')'\n");
		  result = result_f = -1; break;
		}
		int j;
		int open_braces = 1, close_braces = 0;
		for(j = k+1; j < strlen(formula); j++)
		{
		  if(formula[j] == ')') close_braces++;
		  else if(formula[j] == '(') open_braces++;
		  if(open_braces == close_braces) break;
		}
		double tmp_res = 
		  process_sub_formula_f(formula+k, j-k);
		  //process_sub_formula_f(formula+k, strrchr(formula+k, ')')-(formula+k));
		if(strchr(formula+k, '.'))
		  if(number_mode == NUMBER_INT)
		  { result_f = result; number_mode = NUMBER_FLOAT; }
		
		if(number_mode == NUMBER_FLOAT)
		{
		  switch(op)
		  {
		    case '+': result_f += tmp_res; break;
		    case '-': result_f -= tmp_res; break;
		    case '*': result_f *= tmp_res; break;
		    case '/': result_f /= tmp_res; break;
		    case '%': result_f = (int)result_f%(int)tmp_res; break;
		    case '.': result_f = tmp_res; break;
		  }
		}
		else
		{
		  switch(op)
		  {
		    case '+': result += tmp_res; break;
		    case '-': result -= tmp_res; break;
		    case '*': result *= tmp_res; break;
		    case '/': result /= tmp_res; break;
		    case '%': result = result%(int)tmp_res; break;
		    case '.': result = tmp_res; break;
		  }
		}
		k = j;//strrchr(formula+k, ')')-(formula);
		op = formula[k];
		i = k+1;
	      }
	      /////////////////////////////////
	      /////////////////////////////////
	      /////////////////////////////////
	      else if(formula[k] == '+' || formula[k] == '-' ||
		      formula[k] == '*' || formula[k] == '/' ||
		      formula[k] == '%' || formula[k] == '\0')
	      {
		tmp = strndup(formula+i, k-i);
		if(number_mode == NUMBER_FLOAT)
		{
		  switch(op)
		  {
		    case '+':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f += binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f += hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f += octal_to_decimal(tmp);
		      else result_f += atof(tmp);
		      break;
		    case '-':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f -= binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f -= hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f -= octal_to_decimal(tmp);
		      else result_f -= atof(tmp);
		      break;
		    case '*':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f *= binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f *= hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f *= octal_to_decimal(tmp);
		      else result_f *= atof(tmp);
		      break;
		    case '/':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f /= binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f /= hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f /= octal_to_decimal(tmp);
		      else result_f /= atof(tmp);
		      break;
		    case '%':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f = (int)result_f%binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f = (int)result_f%hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f = (int)result_f%octal_to_decimal(tmp);
		      else result_f = (int)result_f%atoi(tmp);
		      break;
		    case '.':
		      if(tmp[0] == 'b' || tmp[0] == 'B')
			result_f = binary_to_decimal(tmp+1);
		      else if((tmp[0] == '0' && tmp[1] == 'X') ||
			      (tmp[0] == '0' && tmp[1] == 'x'))
			result_f = hex_to_decimal(tmp);
		      else if(tmp[0] == '0')
			result_f = octal_to_decimal(tmp);
		      else result_f = atof(tmp);
		      break;
		  }
		}
		else
		{
		  if(strchr(tmp, '.'))
		  {
		    number_mode = NUMBER_FLOAT;
		    if(result) { result_f = result; result = 0; }
		    switch(op)
		    {
		      case '+':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f += binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f += hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f += octal_to_decimal(tmp);
			else result_f += atof(tmp);
			break;
		      case '-':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f -= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f -= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f -= octal_to_decimal(tmp);
			else result_f -= atof(tmp);
			break;
		      case '*':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f *= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f *= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f *= octal_to_decimal(tmp);
			else result_f *= atof(tmp);
			break;
		      case '/':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f /= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f /= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f /= octal_to_decimal(tmp);
			else result_f /= atof(tmp);
			break;
		      case '%':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f = (int)result_f%binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f = (int)result_f%hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f = (int)result_f%octal_to_decimal(tmp);
			else result_f = (int)result_f%atoi(tmp);
			break;
		      case '.':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result_f = binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result_f = hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result_f = octal_to_decimal(tmp);
			else result_f = atof(tmp);
			break;
		    }
		  }
		  else
		  {
		    number_mode = NUMBER_INT;
		    switch(op)
		    {
		      case '+':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result += binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result += hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result += octal_to_decimal(tmp);
			else result += atoi(tmp);
			break;
		      case '-':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result -= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result -= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result -= octal_to_decimal(tmp);
			else result -= atoi(tmp);
			break;
		      case '*':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result *= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result *= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result *= octal_to_decimal(tmp);
			else result *= atoi(tmp);
			break;
		      case '/':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result /= binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result /= hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result /= octal_to_decimal(tmp);
			else result /= atoi(tmp);
			break;
		      case '%':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result = (int)result%binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result = (int)result%hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result = (int)result%octal_to_decimal(tmp);
			else result = (int)result%atoi(tmp);
			break;
		      case '.':
			if(tmp[0] == 'b' || tmp[0] == 'B')
			  result = binary_to_decimal(tmp+1);
			else if((tmp[0] == '0' && tmp[1] == 'X') ||
				(tmp[0] == '0' && tmp[1] == 'x'))
			  result = hex_to_decimal(tmp);
			else if(tmp[0] == '0')
			  result = octal_to_decimal(tmp);
			else result = atoi(tmp);
			break;
		    }
		  }
		}
		op = formula[k];
		i = k+1;
	      }
	      ///////////////////////////////////////
	      //process math functions
	      ///////////////////////////////////////
	      else if(formula[k] >= 'a' && formula[k] <= 'z')
	      {
		char *ob = strchr(formula+k, '(');
		char *cb = strchr(formula+k, ')');
		if(!ob || !cb)
		{
		  if(!ob) printf("Error: missing '('\n");
		  else    printf("Error: missing ')'\n");
		  result = result_f = -1; break;
		}
		double r =
		  process_math_func(formula+k, ob-(formula+k), cb-(formula+k));
		if(r == -1) break;
		if(number_mode == NUMBER_INT)
		{ result_f = result; }
		number_mode = NUMBER_FLOAT;
		switch(op)
		{
		    case '+': result_f += r; break;
		    case '-': result_f -= r; break;
		    case '*': result_f *= r; break;
		    case '/': result_f /= r; break;
		    case '%': result_f = (int)result_f%(int)r; break;
		    case '.': result_f = r; break;
		}
		k = cb-(formula+k)+1;
		op = formula[k];
		i = k+1;
	      }
	      else if(formula[k] == ' ' || formula[k] == '\t') i++;
	      //else if(formula[k] == '#') { COMMENT = 1; break; }
	    }//end for
	    //if(COMMENT) continue;
	    //i = k+1;
	  //}//end while
	  /*if(decimal_point)
	  {
	    char tmp2[20], tmp3[10];
	    tmp2[0] = '%';
	    tmp2[1] = '.';
	    tmp2[2] = '\0';
	    sprintf(tmp3, "%d", decimal_point);
	    strcat(tmp2, tmp3);
	    strcat(tmp2, "f\n");
	    //printf("%s %s\n", tmp3, tmp2);
	    if(number_mode == NUMBER_INT)
	      result_f = result;
	    printf(tmp2, result_f);
	  }
	  else
	  {
	    if(number_mode == NUMBER_INT)
	      printf("%ld\n", result);
	    else
	      printf("%f\n", result_f);
	  }*/
	  //////////////////////////////////////////////
	  //output the result
	  //////////////////////////////////////////////
	  if(output_format == DECIMAL_FORMAT || output_format == ALL_FORMATS)
	  {
	    if(decimal_point)
	    {
	      char tmp2[20], tmp3[10];
	      tmp2[0] = '%';
	      tmp2[1] = '.';
	      tmp2[2] = '\0';
	      sprintf(tmp3, "%d", decimal_point);
	      strcat(tmp2, tmp3);
	      strcat(tmp2, "f\n");
	      //printf("%s %s\n", tmp3, tmp2);
	      if(number_mode == NUMBER_INT)
		result_f = result;
	      printf(tmp2, result_f);
	    }
	    else
	    {
	      if(number_mode == NUMBER_INT) printf("%ld\n", result);
	      else printf("%f\n", result_f);
	    }
	  }
	  if(output_format == BINARY_FORMAT || output_format == ALL_FORMATS)
	  {
	      char *res2;
	      if(number_mode == NUMBER_INT)
	      {
		decimal_to_binary(result, &res2);
		if(res2) { printf("b%s\n", res2); free(res2); }
	      }
	      else
	      {
		decimal_to_binary((long)result_f, &res2);
		if(res2) { printf("b%s\n", res2); free(res2); }	
	      }
	  }
	  if(output_format == OCTAL_FORMAT || output_format == ALL_FORMATS)
	  {
	    if(number_mode == NUMBER_INT)
	      printf("0%o\n", (unsigned)result);
	    else printf("0%o\n", (unsigned int)result_f);
	  }
	  if(output_format == HEX_FORMAT || output_format == ALL_FORMATS)
	  {
	      char *res2;
	      if(number_mode == NUMBER_INT)
	      {
		decimal_to_hex(result, &res2);
		if(res2) { printf("%s\n", res2); free(res2); }
	      }
	      else
	      {
		decimal_to_hex((long)result_f, &res2);
		if(res2) { printf("%s\n", res2); free(res2); }	
	      }
	  }
    }
    return;
}

void show_help()
{
  printf("Commands [followed by ENTER]:\n");
  printf("convert n         Convert n to the output format specified by 'output' below\n");
  printf("decimal=n         How many digits are shown right to decimal point\n");
  printf("format=n          Input format [binary (b), hex (x), decimal (d), octal(o)]\n");
  printf("func              Define a custom function\n");
  printf("funcs             Print a list of custom-defined functions\n");
  printf("help              Show help\n");
  printf("input n           Read a formula file\n");
  printf("math              Print a list of math functions\n");
  printf("output=n          Output format [binary (b), hex (x), decimal (d), octal(o),\n"
	 "                  all formats (a)\n");
  printf("q, quit           Quit\n");
  printf("table n           Print the multiplication table of number 'n'\n");
  printf("ver, version      Show version\n");
}